; Inno Setup script file for PathCopyCopy
; Copyright (c) 2008-2013 Charles Lechasseur
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in
; all copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
; THE SOFTWARE.

#define MyAppName "Path Copy Copy"
#define MyAppVersion "11.1"
#define MyAppFullVersion "11.1"
#define MyAppVerName "Path Copy Copy 11.1"
#define MyAppPublisher "Charles Lechasseur"
#define MyAppURL "http://pathcopycopy.codeplex.com/"
#define MyAppDescription "Path Copy Copy Shell Contextual Menu Extension"
#define MyAppCopyright "(c) 2008-2013, Charles Lechasseur. See LICENSE.TXT for details."
#define MyLicenseFile "..\LICENSE.TXT"

[Setup]
; NOTE: The value of AppId uniquely identifies this application.
; Do not use the same AppId value in installers for other applications.
; (To generate a new GUID, click Tools | Generate GUID inside the IDE.)
AppId={{3C01F274-867C-4D1D-BE8C-CB488C31B0C9}
AppName={#MyAppName}
AppVerName={#MyAppVerName}
AppPublisher={#MyAppPublisher}
AppPublisherURL={#MyAppURL}
AppSupportURL={#MyAppURL}
AppUpdatesURL={#MyAppURL}
DefaultDirName={pf}\{#MyAppName}
DefaultGroupName={#MyAppName}
DisableProgramGroupPage=true
LicenseFile={#MyLicenseFile}
OutputBaseFilename=PathCopyCopy{#MyAppVersion}
Compression=lzma
SolidCompression=true
VersionInfoVersion={#MyAppFullVersion}
VersionInfoDescription={#MyAppDescription}
VersionInfoTextVersion={#MyAppFullVersion}
ArchitecturesInstallIn64BitMode=x64
VersionInfoCopyright={#MyAppCopyright}
DisableReadyMemo=true
AlwaysShowComponentsList=false
DisableReadyPage=true
OutputDir=Output
MinVersion=5.1

[Languages]
Name: english; MessagesFile: compiler:Default.isl

[Files]
Source: ..\bin\Win32\Release\PathCopyCopy.dll; DestDir: {app}; Flags: ignoreversion regserver restartreplace overwritereadonly uninsrestartdelete uninsremovereadonly 32bit; DestName: PCC32.dll
Source: ..\bin\x64\Release\PathCopyCopy.dll; DestDir: {app}; Flags: ignoreversion regserver restartreplace overwritereadonly uninsrestartdelete uninsremovereadonly 64bit; DestName: PCC64.dll; Check: Is64BitInstallMode
Source: ..\bin\Win32\Release\PathCopyCopySettings.exe; DestDir: {app}; Flags: ignoreversion restartreplace overwritereadonly uninsrestartdelete uninsremovereadonly
Source: ..\bin\Win32\Release\PathCopyCopyRegexTester.exe; DestDir: {app}; Flags: ignoreversion restartreplace overwritereadonly uninsrestartdelete uninsremovereadonly
Source: ..\LICENSE.TXT; DestDir: {app}; Flags: overwritereadonly uninsremovereadonly
Source: ..\HISTORY.TXT; DestDir: {app}; Flags: overwritereadonly uninsremovereadonly
Source: .\Input\Path Copy Copy on CodePlex.url; DestDir: {app}; Flags: overwritereadonly uninsremovereadonly
Source: ..\Schemas\PipelinePluginCollection.xsd; DestDir: {app}\Schemas; Flags: overwritereadonly uninsremovereadonly
Source: ..\Samples\SampleCOMPluginCpp\SampleCOMPlugin.sln; DestDir: {app}\Samples\Plugins\COM\C++
Source: ..\Samples\SampleCOMPluginCpp\SampleCOMPlugin\*; DestDir: {app}\Samples\Plugins\COM\C++\SampleCOMPlugin; Excludes: dlldata.c,SampleCOMPlugin_i.h,SampleCOMPlugin_i.c,SampleCOMPlugin_p.c,*.aps,*.user,*.ncb,*.suo; Languages: 

[Icons]
Name: {group}\Settings; Filename: {app}\PathCopyCopySettings.exe; Parameters: {code:SettingsShortcutParams}; Flags: excludefromshowinnewinstall

[Registry]
Root: HKLM32; Subkey: Software\clechasseur\PathCopyCopy\PipelinePlugins; Flags: uninsdeletekeyifempty 
Root: HKLM32; Subkey: Software\clechasseur\PathCopyCopy\Plugins; Flags: uninsdeletekeyifempty
Root: HKLM32; Subkey: Software\clechasseur\PathCopyCopy; Flags: uninsdeletekeyifempty
Root: HKLM64; Subkey: Software\clechasseur\PathCopyCopy\PipelinePlugins; Flags: uninsdeletekeyifempty; Check: IsWin64
Root: HKLM64; Subkey: Software\clechasseur\PathCopyCopy\Plugins; Flags: uninsdeletekeyifempty; Check: IsWin64
Root: HKLM64; Subkey: Software\clechasseur\PathCopyCopy; Flags: uninsdeletekeyifempty; Check: IsWin64

[Code]
const
  CDwordSwitch = '/PCCREGVALUEDWORD=';
  CStringSwitch = '/PCCREGVALUESZ=';
  CPipelinePluginSwitch = '/PCCREGPIPELINEPLUGIN=';
  CPipelinePluginsDisplayOrderSwitch = '/PCCREGPIPELINEPLUGINSDISPLAYORDER=';
  CStringSeparator = ',';
  
  CCommonCommandsChoice = 0;
  CNetworkCommandsChoice = 1;
  CAllCommandsChoice = 2;
  
var
  GCommandsPage: TInputOptionWizardPage;
  GIsUpgrade: Boolean;
  
// This procedure will perform a revise of the global PCC registry key
// to account for changes in config values. This is a bit similar to
// the C++ class PCC::Settings::ApplyRevisions() except we don't really
// identify revision numbers.
procedure RevisePCCRegistry(const ARegKeyPath: string);
var
  Value: string;
begin
  // Revise: See PCC::Settings::ApplyNewPluginOrganizationRevision201201311()
  if not RegValueExists(HKEY_LOCAL_MACHINE, ARegKeyPath, 'PluginsNotInSubmenu') then
    if RegQueryStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'DisabledPlugins', Value) then
      if RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'PluginsNotInSubmenu', Value) then
        RegDeleteValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'DisabledPlugins');
  if not RegValueExists(HKEY_LOCAL_MACHINE, ARegKeyPath, 'PluginsInMainMenu') then
    if RegQueryStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'DefaultPlugin', Value) then
      if RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'PluginsInMainMenu', Value) then
        RegDeleteValue(HKEY_LOCAL_MACHINE, ARegKeyPath, 'DefaultPlugin');
end;

// This procedure scans the command-line for values to write
// to the settings key in HKLM.
procedure CreatePCCRegValues(const ARegKeyPath: string);
var
  I: Integer;
  S, V, N: string;
  VI: Integer;
begin
  // Scan command-line arguments to find ours.
  for I := 1 to ParamCount do
  begin
    S := ParamStr(I);
    if Pos(CDwordSwitch, S) = 1 then
    begin
      // Value follows.
      S := Copy(S, Length(CDwordSwitch) + 1, MaxInt);
      
      // Remove quotes if needed.
      if (S[1] = '"') and (S[Length(S)] = '"') then
        S := RemoveQuotes(S);
        
      // String begins with the value name and a separator is used for the value.
      VI := StrToInt(Copy(S, Pos(CStringSeparator, S) + 1, MaxInt));
      S := Copy(S, 1, Pos(CStringSeparator, S) - 1);
      
      // Save in registry.
      RegWriteDWordValue(HKEY_LOCAL_MACHINE, ARegKeyPath, S, VI);
    end
    else if Pos(CStringSwitch, S) = 1 then
    begin
      // Value follows.
      S := Copy(S, Length(CStringSwitch) + 1, MaxInt);
      
      // Remove quotes if needed.
      if (S[1] = '"') and (S[Length(S)] = '"') then
        S := RemoveQuotes(S);
        
      // String begins with the value name and a separator is used for the value.
      V := Copy(S, Pos(CStringSeparator, S) + 1, MaxInt);
      S := Copy(S, 1, Pos(CStringSeparator, S) - 1);
        
       // Save in registry.
       RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath, S, V);
    end
    else if Pos(CPipelinePluginSwitch, S) = 1 then
    begin
      // Data follows.
      S := Copy(S, Length(CPipelinePluginSwitch) + 1, MaxInt);
      
      // Remove quotes if needed.
      if (S[1] = '"') and (S[Length(S)] = '"') then
        S := RemoveQuotes(S);
        
      // In order, the string contains: plugin ID, pipeline and description.
      V := Copy(S, Pos(CStringSeparator, S) + 1, MaxInt);
      N := Copy(V, Pos(CStringSeparator, V) + 1, MaxInt);
      V := Copy(V, 1, Pos(CStringSeparator, V) - 1);
      S := Copy(S, 1, Pos(CStringSeparator, S) - 1);
      
      // Create key for the plugin.
      RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath +
        '\PipelinePlugins\' + S, '', V);
      RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath +
        '\PipelinePlugins\' + S, 'Description', N);
    end
    else if Pos(CPipelinePluginsDisplayOrderSwitch, S) = 1 then
    begin
      // Display order follows.
      S := Copy(S, Length(CPipelinePluginsDisplayOrderSwitch) + 1, MaxInt);
      
      // Remove quotes if needed.
      if (S[1] = '"') and (S[Length(S)] = '"') then
        S := RemoveQuotes(S);
        
      // Display order is a comma-separated list of plugin IDs. We don't
      // have to pre-process them, merely write them to the registry.
      RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath + '\PipelinePlugins',
        'DisplayOrder', S);
    end;
  end;
end;

// This procedure uses the user choice of the "commands"
// page to pre-configure command states in the registry.
procedure ConfigureCommandsStates(const ARegKeyPath: string);
var
  PluginsNotInSubmenu: string;
begin
  // The registry setting allowing us to do this is actually a list
  // of plugins *not* to include in the submenu. We need to build this list
  // depending on the user choice. For a list of IDs used in this procedure,
  // please refer to:
  // http://pathcopycopy.codeplex.com/wikipage?title=Network%20Administrator
  PluginsNotInSubmenu := '';
  if not GCommandsPage.Values[CAllCommandsChoice] then
  begin
    // Remove these commands if we don't show them all: Short Name,
    // Short Path, Short Parent Folder Path, Short UNC Path,
    // Short UNC Parent Folder Path, Unix Path, Cygwin Path.
    PluginsNotInSubmenu := '{C8757DCA-1BA8-48d0-8145-B93BBB9237AF},' +
      '{93B362D1-2B13-467a-B618-6784F24F3BA1},' +
      '{0929538B-5F52-4480-9902-2BBEC9C84905},' +
      '{891F9E3E-0B70-49fd-A86C-53B21C6193C7},' +
      '{73188FB3-8E14-409c-95EF-BA608FDC1274},' +
      '{5B5DA5CB-3284-45a9-A1E5-4D6B03107924},' +
      '{CD50DCE3-9A5C-4adf-B552-1741361567D6}';
    if not GCommandsPage.Values[CNetworkCommandsChoice] then
    begin
      // Disable the network commands: Long UNC Path,
      // Long UNC Parent Folder Path, Internet Path.
      PluginsNotInSubmenu := PluginsNotInSubmenu + ',' +
        '{EA24BCC3-35F5-46b9-A2A5-A12A0AED2D28},' +
        '{E2C942AC-917C-4aee-A867-8F6AB960BA76},' +
        '{8F2ADCCC-9693-407d-9300-FCCB9A12B982}';
    end;
  end;
  RegWriteStringValue(HKEY_LOCAL_MACHINE, ARegKeyPath,
    'PluginsNotInSubmenu', PluginsNotInSubmenu);
end;

// Called when setup first initializes. Note that the
// wizard form does not exist at that point.
function InitializeSetup: Boolean;
begin
  // Check if this is an upgrade by checking the main DLL's
  // AppID registry entry. (We could also check the installer's
  // AppID in the Uninstall key.)
  GIsUpgrade := RegKeyExists(HKEY_CLASSES_ROOT,
    'AppID\{44F7E5A2-1286-45F5-9A7A-A95A41B72918}');
  Result := True;
end;

// Called after the wizard form has been created.
// We can use this opportunity to add pages to it.
procedure InitializeWizard;
begin
  // Create a page to allow the user to select which commands
  // to include in the submenu by default.
  GCommandsPage := CreateInputOptionPage(wpSelectTasks,
    'Configuration', 'Pre-configure commands',
    'Path Copy Copy ships with multiple commands used to copy paths ' +
    '(long path, network path, etc.) Please choose which ones to include ' +
    'by default. This can be changed later via Settings.', True, False);
  GCommandsPage.Add('Commonly-used commands only');
  GCommandsPage.Add('Common commands and commands for network environments');
  GCommandsPage.Add('All commands, including power-user commands');
  GCommandsPage.Values[CCommonCommandsChoice] := True;
end;

// Called for each wizard page.
// Returning True will skip that page.
function ShouldSkipPage(PageID: Integer): Boolean;
begin
  // Skip our "commands" page if this is an upgrade.
  // We do this to avoid overwriting existing settings and
  // since it's difficult to parse the "PluginsNotInSubmenu"
  // registry value, which can be modified by other means.
  Result := (PageID = GCommandsPage.ID) and GIsUpgrade;
end;

// Called during several installation steps. We'll use the
// opportunity to write admin registry keys at the end.
procedure CurStepChanged(CurStep: TSetupStep);
begin
  if CurStep = ssPostInstall then
  begin
    // If this is not an upgrade, configure commands states.
    // Do this before setting registry values, this way the
    // command-line parameters will take precedence.
    if not GIsUpgrade then
    begin
      ConfigureCommandsStates('Software\clechasseur\PathCopyCopy');
      if Is64BitInstallMode then
      begin
        // Also configure commands for the 32-bit version.
        ConfigureCommandsStates('Software\Wow6432Node\clechasseur\PathCopyCopy');
      end;
    end;
    
    // Now process reg values on the command-line.  
    CreatePCCRegValues('Software\clechasseur\PathCopyCopy');
    if Is64BitInstallMode then
    begin
      // Also save the same values for the 32-bit version.
      CreatePCCRegValues('Software\Wow6432Node\clechasseur\PathCopyCopy');
    end;
    
    // Revise any registry values in old formats. We perform this *after*
    // processing command-line arguments, so that old-style command-line
    // values will get upgraded, allowing network admins to upgrade
    // without fear of their values no longer having an effect.
    RevisePCCRegistry('Software\clechasseur\PathCopyCopy');
    if Is64BitInstallMode then
    begin
      // Also revise the 32-bit version.
      RevisePCCRegistry('Software\Wow6432Node\clechasseur\PathCopyCopy');
    end;
  end;
end;

// Called to know the parameters to pass to the Settings app
// when invoked from the shortcut in the Windows Start menu.
function SettingsShortcutParams(Params: string): string;
begin
  // First argument is whether we're running on a 64-bit system.
  if Is64BitInstallMode then
    Result := '1'
  else
    Result := '0';
    
  // Second parameter is whether we run under WOW64. This is never
  // true when launched from the Start menu, so no need to specify it.
end;
