// COMPlugin.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <Plugin.h>
#include <PathCopyCopy_i.h>
#include <exception>


namespace PCC
{
    namespace Plugins
    {
        //
        // COMPlugin
        //
        // Plugin that wraps a COM plugin implementation.
        //
        class COMPlugin : public PCC::Plugin
        {
        public:
            explicit                COMPlugin(const CLSID& p_CLSID);

            virtual const GUID&     Id() const;
            ULONG                   GroupId() const;
            ULONG                   GroupPosition() const;

            virtual std::wstring    Description() const;
            virtual std::wstring    HelpText() const;
            virtual bool            Enabled(const std::wstring& p_ParentPath,
                                            const std::wstring& p_File) const;

            virtual std::wstring    GetPath(const std::wstring& p_File) const;

        private:
            GUID                    m_Id;               // Unique plugin ID.
            CComPtr<IPathCopyCopyPlugin>
                                    m_cpPlugin;         // Pointer to COM plugin implementation.
            CComQIPtr<IPathCopyCopyPluginGroupInfo>
                                    m_cpPluginGroup;    // Reference to grouping interface implementation. Can be NULL.
            CComQIPtr<IPathCopyCopyPluginStateInfo>
                                    m_cpPluginState;    // Reference to state changing interface implementation. Can be NULL.
            std::wstring            m_Description;      // Plugin description.

            // THESE METHODS ARE NOT IMPLEMENTED.
                                    COMPlugin(const COMPlugin&);
            COMPlugin&              operator=(const COMPlugin&);
        };

        //
        // COMPluginError
        //
        // Exception class thrown when a COMPlugin encounters an error.
        //
        class COMPluginError : public std::exception
        {
        public:
            explicit                COMPluginError(const HRESULT p_Result);
                                    COMPluginError(const COMPluginError& p_Exception);
            virtual                 ~COMPluginError();

            COMPluginError&         operator=(const COMPluginError& p_Exception);

            virtual const char*     what() const;

        private:
            HRESULT                 m_Result;   // Contains an HRESULT returned by the call.
        };

    } // namespace Plugins

} // namespace PCC
