// PipelinePlugin.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <Plugin.h>
#include <PluginPipeline.h>


namespace PCC
{
    namespace Plugins
    {
        //
        // PipelinePlugin
        //
        // Plugin that loads an encoded pipeline of elements that can all
        // modify a given path and executes each element on any path received.
        //
        class PipelinePlugin : public PCC::Plugin
        {
        public:
            explicit                    PipelinePlugin(const GUID& p_PluginId,
                                                       const std::wstring& p_PluginDescription,
                                                       const std::wstring& p_EncodedElements);

            virtual const GUID&         Id() const;

            virtual std::wstring        Description() const;
            virtual bool                Enabled(const std::wstring& p_ParentPath,
                                                const std::wstring& p_File) const;

            virtual std::wstring        GetPath(const std::wstring& p_File) const;

        private:
            GUID                        m_Id;           // Plugin ID.
            std::wstring                m_Description;  // Plugin description.
            PCC::PipelineSP             m_spPipeline;   // Pipeline to execute on each path received.

            // NoDoc: THESE METHODS ARE NOT IMPLEMENTED
                                        PipelinePlugin(const PipelinePlugin&);
            PipelinePlugin&             operator=(const PipelinePlugin&);
        };

    } // namespace Plugins

} // namespace PCC
