// ShortPathPlugin.cpp
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#include "stdafx.h"
#include "ShortPathPlugin.h"

#include "resource.h"


namespace
{
    // Plugin unique ID: {93B362D1-2B13-467a-B618-6784F24F3BA1}
    const GUID SHORT_PATH_PLUGIN_ID = { 0x93b362d1, 0x2b13, 0x467a, { 0xb6, 0x18, 0x67, 0x84, 0xf2, 0x4f, 0x3b, 0xa1 } };

} // anonymous namespace

namespace PCC
{
    namespace Plugins
    {
        //
        // Constructor.
        //
        ShortPathPlugin::ShortPathPlugin()
            : InternalPlugin(IDS_SHORT_PATH_PLUGIN_DESCRIPTION, IDS_SHORT_PATH_PLUGIN_HINT)
        {
        }

        //
        // Returns the plugin's unique identifier.
        //
        // @return Unique identifier.
        //
        const GUID& ShortPathPlugin::Id() const
        {
            return SHORT_PATH_PLUGIN_ID;
        }

        //
        // Returns the short path of the specified file.
        //
        // @param p_File File path.
        // @returns Short path.
        //
        std::wstring ShortPathPlugin::GetPath(const std::wstring& p_File) const
        {
            wchar_t shortPath[MAX_PATH + 1];
            DWORD copied = ::GetShortPathNameW(p_File.c_str(),
                                               shortPath,
                                               sizeof(shortPath) / sizeof(wchar_t));
            return std::wstring(shortPath, copied);
        }

        //
        // Protected constructor with custom description and help text resources.
        //
        // @param p_DescriptionStringResourceID ID of description resource.
        // @param p_HelpTextStringResourceID ID of help text resource.
        //
        ShortPathPlugin::ShortPathPlugin(const unsigned short p_DescriptionStringResourceID,
                                         const unsigned short p_HelpTextStringResourceID)
            : InternalPlugin(p_DescriptionStringResourceID, p_HelpTextStringResourceID)
        {
        }

    } // namespace Plugins

} // namespace PCC
