// ShortUNCPathPlugin.cpp
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.


#include "stdafx.h"
#include "ShortUNCPathPlugin.h"

#include "resource.h"


namespace
{
    // Plugin unique ID: {891F9E3E-0B70-49fd-A86C-53B21C6193C7}
    const GUID SHORT_UNC_PATH_PLUGIN_ID = { 0x891f9e3e, 0xb70, 0x49fd, { 0xa8, 0x6c, 0x53, 0xb2, 0x1c, 0x61, 0x93, 0xc7 } };

} // anonymous namespace

namespace PCC
{
    namespace Plugins
    {
        //
        // Constructor.
        //
        ShortUNCPathPlugin::ShortUNCPathPlugin()
            : LongUNCPathPlugin(IDS_SHORT_UNC_PATH_PLUGIN_DESCRIPTION, IDS_SHORT_UNC_PATH_PLUGIN_HINT)
        {
        }

        //
        // Returns the plugin's unique identifier.
        //
        // @return Unique identifier.
        //
        const GUID& ShortUNCPathPlugin::Id() const
        {
            return SHORT_UNC_PATH_PLUGIN_ID;
        }

        //
        // Returns the short UNC path of the specified file.
        //
        // @param p_File File path.
        // @returns Short UNC path.
        //
        std::wstring ShortUNCPathPlugin::GetPath(const std::wstring& p_File) const
        {
            // First call inherited to get a long path.
            std::wstring longPath = LongUNCPathPlugin::GetPath(p_File);

            // Now ask for a short version and return it.
            wchar_t shortPath[MAX_PATH + 1];
            DWORD copied = ::GetShortPathNameW(longPath.c_str(),
                                               shortPath,
                                               sizeof(shortPath) / sizeof(wchar_t));
            return std::wstring(shortPath, copied);
        }

    } // namespace Plugins

} // namespace PCC
