// PathCopyCopyPluginsRegistration.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <CriticalSection.h>

#include <COMPlugin.h>
#include "PathCopyCopySettings.h"
#include <PluginRegistry.h>


namespace PCC
{
    //
    // PluginsRegistration
    //
    // Static class responsible for registering the PCC plugins
    // in the PluginRegistry.
    //
    class PluginsRegistration
    {
    public:
        static void     Register(PluginRegistry& p_rRegistry,
                                 Settings& p_rSettings);

    private:
        typedef std::pair<ULONG, ULONG>                     GroupIdPositionPair;        // Pair storing a group ID and position for a plugin.
        typedef std::shared_ptr<Plugins::COMPlugin>         COMPluginSP;                // Reference to a COM plugin.
        typedef std::pair<COMPluginSP, GroupIdPositionPair> COMPluginSPGroupInfoPair;   // Pair storing a COM plugin with associated group info.
        typedef std::vector<COMPluginSPGroupInfoPair>       COMPluginSPGroupInfoPairV;  // Vector of COM plugins with associated group info.

        static void     RegisterCOMPlugins(const PluginSP& p_spSeparator,
                                           PluginRegistry& p_rRegistry,
                                           Settings&       p_rSettings);

        static void     RegisterPipelinePlugins(const PluginSP& p_spSeparator,
                                                PluginRegistry& p_rRegistry,
                                                Settings&       p_rSettings);

        //
        // COMPluginGroupInfoLess
        //
        // Predicate to sort COM plugins with group info by ascending
        // group ID and position.
        //
        struct COMPluginGroupInfoLess {
            bool        operator()(const COMPluginSPGroupInfoPair& p_Plugin1,
                                   const COMPluginSPGroupInfoPair& p_Plugin2);
        };

        // THESE METHOD ARE NOT IMPLEMENTED
                        PluginsRegistration();
                        ~PluginsRegistration();
    };

} // namespace PCC
