// PathCopyCopySettings.h
// (c) 2009-2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <UserOverrideableRegKey.h>

#include <exception>


namespace PCC
{
    //
    // Settings
    //
    // Class used to access the PathCopyCopy settings, be it per-user or globals.
    //
    // This class is not thread-safe. Each thread should create its own copy.
    //
    class Settings
    {
    public:
                        Settings();

        bool            GetUseHiddenShares();
        bool            GetAddQuotesAroundPaths();
        bool            GetUseIconForDefaultPlugin();
        bool            GetUseIconForSubmenu();
        bool            GetAlwaysShowSubmenu();
        bool            GetPluginsNotInSubmenu(CLSIDV& p_rvPluginIds);
        bool            GetPluginsInMainMenu(CLSIDV& p_rvPluginIds);

        bool            NeedsUpdateCheck();
        void            SetLastUpdateCheckNow();

        bool            GetEditingDisabled();

        void            LastPluginUpdateId(GUID& p_rUpdateId);

        void            GetCOMPlugins(CLSIDV& p_rvCLSIDs);
        bool            RegisterCOMPlugin(const CLSID& p_CLSID);
        bool            UnregisterCOMPlugin(const CLSID& p_CLSID);

        void            GetPipelinePlugins(PluginSPV& p_rvspPlugins,
                                           const bool p_AddSeparator = true);

    private:
        UserOverrideableRegKey
                        m_UserKey;              // PCC user settings registry key.
        UserOverrideableRegKey
                        m_PipelinePluginsKey;   // PCC user pipeline plugins registry key.
        CRegKey         m_PluginsKey;           // PCC plugins registry key.
        bool            m_PluginsKeyReadOnly;   // Whether the plugins key is read-only.

        std::wstring    GetCOMPluginInfo(const CLSID& p_CLSID) const;

        void            ApplyRevisions();
        void            ApplyNewPluginOrganizationRevision201201311();
        void            ApplyPluginsInMainMenuAdjustment201204051();

        // THESE METHOD ARE NOT IMPLEMENTED
                        Settings(const Settings&);
        Settings&       operator=(const Settings&);
    };

    //
    // SettingsException
    //
    // Exception class used to indicate an error accessing the PathCopyCopy registry settings.
    //
    class SettingsException : public std::exception
    {
    public:
                        SettingsException(const LONG p_ErrorCode);
        virtual         ~SettingsException();

        LONG            ErrorCode() const;

        virtual const char*
                        what() const;

    private:
        LONG            m_ErrorCode;        // The Windows error code.
    };
}
