// PluginPipeline.h
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <vector>


namespace PCC
{
    // Forward declarations
    class PipelineElement;
    typedef std::shared_ptr<PipelineElement> PipelineElementSP;
    typedef std::vector<PipelineElementSP> PipelineElementSPV;
    class Pipeline;
    typedef std::shared_ptr<Pipeline> PipelineSP;

    //
    // Pipeline
    //
    // Class representing a pipeline in which each element can apply
    // changes to a path. Meant to be used by custom path plugins.
    //
    class Pipeline
    {
    public:
        explicit        Pipeline(const PipelineElementSPV& p_vspElements);
        explicit        Pipeline(const std::wstring& p_EncodedElements);
        virtual         ~Pipeline();

        void            ModifyPath(std::wstring& p_rPath) const;
        bool            ShouldBeEnabledFor(const std::wstring& p_ParentPath,
                                           const std::wstring& p_File) const;

    private:
        PipelineElementSPV
                        m_vspElements;      // Elements in the pipeline.

        // THESE METHODS ARE NOT IMPLEMENTED
                        Pipeline(const Pipeline&);
        Pipeline&       operator=(const Pipeline&);
    };

    //
    // PipelineElement
    //
    // Element in a pipeline. Each element can apply its own modification
    // to a path.
    //
    class PipelineElement
    {
    public:
                        PipelineElement();
        virtual         ~PipelineElement();

        virtual void    ModifyPath(std::wstring& p_rPath) const = 0;
        virtual bool    ShouldBeEnabledFor(const std::wstring& p_ParentPath,
                                           const std::wstring& p_File) const;
    };

} // namespace PCC
