// PluginUtils.h
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <CriticalSection.h>
#include <UserOverrideableRegKey.h>

#include <regex>


namespace PCC
{
    //
    // PluginUtils
    //
    // Static class with methods designed to help plugin writers.
    //
    class PluginUtils
    {
    public:
        static bool     ExtractFolderFromPath(std::wstring& p_rPath);

        static bool     IsUNCPath(const std::wstring& p_FilePath);
        static bool     GetMappedDriveFilePath(std::wstring& p_rFilePath);
        static bool     GetNetworkShareFilePath(std::wstring& p_rFilePath,
                                                const bool p_UseHiddenShares);
        static bool     GetHiddenDriveShareFilePath(std::wstring& p_rFilePath);
        static const std::wstring&
                        GetLocalComputerName();

        static long     ReadRegistryStringValue(HKEY const p_hKey,
                                                const wchar_t* const p_pValueName,
                                                std::wstring& p_rValue);
        static long     ReadRegistryStringValue(UserOverrideableRegKey& p_rKey,
                                                const wchar_t* const p_pValueName,
                                                std::wstring& p_rValue);

        static void     GetMultiStringLineBeginningWith(const std::wstring& p_MultiStringValue,
                                                        const std::wstring& p_Prefix,
                                                        std::wstring& p_rLine);
        static void     StringToCLSIDs(std::wstring& p_rClsidsAsString,
                                       const wchar_t p_Separator,
                                       CLSIDV& p_rvClsids);
        static void     StringToUInts(std::wstring& p_rUIntsAsString,
                                        const wchar_t p_Separator,
                                        UIntV& p_rvUInts);
        static std::wstring
                        UIntsToString(const UIntV& p_vUInts,
                                      const wchar_t p_Separator);

    private:
        static CriticalSection
                        s_Lock;                     // Critical section to protect member access.
        static std::wstring
                        s_ComputerName;             // Name of local computer.
        static bool     s_HasComputerName;          // Whether we have precomputed the computer name.
        static std::wregex
                        s_HiddenDriveShareRegex;    // Regex used to perform conversion of hidden drive shares.

        // THESE METHODS ARE NOT IMPLEMENTED.
                        PluginUtils();
                        ~PluginUtils();
    };

} // namespace PCC
