// StCursor.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <windows.h>


//
// StCursor
//
// Shows a cursor during lifecycle and restores previous cursor when destroyed.
//
class StCursor
{
public:
                        //
                        // Constructor.
                        // Loads a cursor and displays it, saving the old cursor.
                        //
                        // @param p_hModule Handle of module containing cursor resource.
                        //                  To load a system cursor, pass NULL.
                        // @param p_pCursorName Name of cursor resource to load.
                        //                      Can be a resource ID (using MAKEINTRESOURCE)
                        //                      or an IDC_ const value for system cursors.
                        //
                        StCursor(HINSTANCE p_hModule, LPCTSTR p_pCursorName)
                            : m_hPreviousCursor(0)
                        {
                            HCURSOR hNewCursor = ::LoadCursor(p_hModule, p_pCursorName);
                            m_hPreviousCursor = ::SetCursor(hNewCursor);
                        }

                        //
                        // Destructor.
                        // Restores the cursor saved in the constructor.
                        //
                        ~StCursor()
                        {
                            ::SetCursor(m_hPreviousCursor);
                        }

private:
    HCURSOR             m_hPreviousCursor;      // Handle of previous cursor.

    // THESE METHODS ARE NOT IMPLEMENTED.
                        StCursor(const StCursor&);
    StCursor&           operator=(const StCursor&);
};
