// UserOverrideableRegKey.h
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include <vector>


//
// UserOverrideableRegKey
//
// Wrapper for a registry key that exists in both CURRENT_USER and LOCAL_MACHINE.
// Values in HKCU will override those found in HKLM.
//
class UserOverrideableRegKey
{
public:
    // Info about a subkey of this overridable registry key.
    struct SubkeyInfo {
        HKEY            m_hParent;      // Parent of this subkey.
        std::wstring    m_KeyName;      // Name of the subkey.

                        SubkeyInfo();
                        SubkeyInfo(HKEY const p_hParent,
                                   const wchar_t* const p_pKeyName);
    };
    typedef std::vector<SubkeyInfo> SubkeyInfoV;

    explicit            UserOverrideableRegKey(const wchar_t* const p_pKeyPath,
                                               const wchar_t* const p_pUserKeyPath = 0);

    bool                Locked();

    long                QueryDWORDValue(const wchar_t* const p_pValueName,
                                        DWORD& p_rValue);
    long                QueryQWORDValue(const wchar_t* const p_pValueName,
                                        ULONGLONG& p_rValue);
    long                QueryGUIDValue(const wchar_t* const p_pValueName,
                                       GUID& p_rValue);
    long                QueryValue(const wchar_t* const p_pValueName,
                                   DWORD* const p_pValueType,
                                   void* const p_pValue,
                                   DWORD* const p_pValueSize);

    void                GetSubKeys(SubkeyInfoV& p_rvSubkeys);

    long                SetQWORDValue(const wchar_t* const p_pValueName,
                                      const ULONGLONG p_Value);
    long                SetGUIDValue(const wchar_t* const p_pValueName,
                                     const GUID& p_Value);
    long                SetStringValue(const wchar_t* const p_pValueName,
                                       const wchar_t* const p_pValue);

    long                DeleteValue(const wchar_t* const p_pValueName);

private:
    CRegKey             m_GlobalKey;        // Wrapper for the global key in HKLM.
    CRegKey             m_UserKey;          // Wrapper for user key in HKCU.

    static void         GetSubKeys(CRegKey& p_rKey,
                                   SubkeyInfoV& p_rvSubkeys);

    // THESE METHODS ARE NOT IMPLEMENTED
                        UserOverrideableRegKey(const UserOverrideableRegKey&);
    UserOverrideableRegKey&
                        operator=(const UserOverrideableRegKey&);
};
