// PathCopyCopyTypes.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

// This header contains definition of common types used throughout the project.

#include "Plugin.h"

#include <memory>
#include <vector>
#include <string>


namespace PCC
{
    // Forward declarations.
    class Plugin;

    typedef std::vector<std::wstring>       WStringV;   // Vector of strings.
    typedef std::vector<CLSID>              CLSIDV;     // Vector of class IDs.
    typedef std::vector<unsigned long>      UIntV;      // Vector of unsigned integers.

    typedef WStringV                        FilesV;     // Vector of file paths.

    typedef std::shared_ptr<Plugin>         PluginSP;   // Shared pointer to a Plugin object.
    typedef std::vector<PluginSP>           PluginSPV;  // Vector of Plugin (via shared pointers).

} // namespace PCC

namespace std
{
    //
    // Predicate used to sort CLSIDs so that we can find them more easily.
    //
    template<>
    struct less<CLSID> {
        bool operator()(const CLSID& p_Left, const CLSID& p_Right) const {
            return ::memcmp(&p_Left, &p_Right, sizeof(CLSID)) < 0;
        }
    };
}

// Macro that includes code in debug only.
#ifdef _DEBUG
#   define PCCDEBUGCODE(C) C
#else
#   define PCCDEBUGCODE(C)
#endif // _DEBUG
