// PluginRegistry.h
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#pragma once

#include "PathCopyCopyTypes.h"


namespace PCC
{
    //
    // PluginRegistry
    //
    // Container responsible for managing PCC plugins. Can be used to register
    // plugins as well as fetch plugins at runtime.
    //
    // This class is NOT thread-safe. Each user should create its own copy.
    //
    class PluginRegistry
    {
    public:
                        PluginRegistry();

        void            Register(const PluginSP& p_spPlugin);
        void            Clear();

        size_t          PluginCount() const;
        const PluginSPV&
                        Plugins() const;
        PluginSP        FindPlugin(const GUID& p_PluginId) const;

        static void     SetLocalRegistry(const PluginRegistry* const p_pRegistry);
        static void     ResetLocalRegistry(const PluginRegistry* const p_pRegistry);
        static const PluginRegistry*
                        GetLocalRegistry();

    private:
        PluginSPV       m_vspPlugins;   // Vector storing registered plugins.


        // THESE METHOD ARE NOT IMPLEMENTED
                        PluginRegistry(const PluginRegistry&);
        PluginRegistry& operator=(const PluginRegistry&);
    };

} // namespace PCC
