// CriticalSection.cpp
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#include "stdafx.h"
#include "CriticalSection.h"


//
// Constructor. Initializes the critical section.
//
CriticalSection::CriticalSection()
    : m_Impl()
{
    ::InitializeCriticalSection(&m_Impl);
}

//
// Destructor. Destroys the critical section. Make sure it is no longer in use.
//
CriticalSection::~CriticalSection()
{
    ::DeleteCriticalSection(&m_Impl);
}

//
// Acquires a lock on the critical section for the calling thread.
// If another thread currently has a lock, the calling thread blocks until the lock is released.
//
void CriticalSection::Lock()
{
    ::EnterCriticalSection(&m_Impl);
}

//
// Releases a lock on the critical section acquired by the calling thread via Lock().
//
void CriticalSection::Unlock()
{
    ::LeaveCriticalSection(&m_Impl);
}
