// PathCopyCopyConfigHelper.cpp
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#include "stdafx.h"
#include "PathCopyCopyConfigHelper.h"


// CPathCopyCopyConfigHelper

//
// Constructor.
//
CPathCopyCopyConfigHelper::CPathCopyCopyConfigHelper()
    : m_Registry()
{
}

//
// IPathCopyCopyConfigHelper::PluginCount
//
// Returns the number of plugins in the registry.
//
// @param p_pCount Will contain the number of plugins in the registry.
// @return Result code.
//
STDMETHODIMP CPathCopyCopyConfigHelper::get_PluginCount(ULONG *p_pCount)
{
    HRESULT hRes = S_OK;

    if (p_pCount != 0) {
        *p_pCount = static_cast<ULONG>(m_Registry.PluginCount());
    } else {
        hRes = E_INVALIDARG;
    }

    return hRes;
}

//
// IPathCopyCopyConfigHelper::GetPluginInfo
//
// Returns information about a plugin in the registry.
//
// @param p_Index 0-based index of the plugin requested.
// @param p_ppId Will contain a string representation of the plugin ID.
// @param p_ppDescription Will contain the plugin's description (what will appear in the contextual menu).
// @param p_pIsSeparator Will contain VARIANT_TRUE if this is a separator plugin, otherwise VARIANT_FALSE.
// @return Result code.
//
STDMETHODIMP CPathCopyCopyConfigHelper::GetPluginInfo(ULONG p_Index, BSTR *p_ppId, BSTR *p_ppDescription, VARIANT_BOOL *p_pIsSeparator)
{
    HRESULT hRes = S_OK;

    if (p_ppId != 0 && p_ppDescription != 0 && p_Index < m_Registry.PluginCount()) {
        PCC::PluginSP spPlugin = m_Registry.Plugins()[p_Index];
        OLECHAR pluginId[100];
        if (::StringFromGUID2(spPlugin->Id(), pluginId, 100)) {
            *p_ppId = ::SysAllocString(pluginId);
            *p_ppDescription = ::SysAllocString(spPlugin->Description().c_str());
            if (p_pIsSeparator != 0) {
                *p_pIsSeparator = spPlugin->IsSeparator() ? VARIANT_TRUE : VARIANT_FALSE;
            }
        } else {
            hRes = E_FAIL;
        }
    } else {
        hRes = E_INVALIDARG;
    }

    return hRes;
}
