// Plugin.cpp
// (c) 2008, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#include "StdAfx.h"
#include "Plugin.h"


namespace PCC
{
    //
    // Destructor.
    //
    Plugin::~Plugin()
    {
    }

    //
    // Returns the help text to show in the Windows Explorer status bar
    // when the user hovers the cursor over the plugin's menu item.
    // If not overridden, no help text will be shown.
    //
    // @return Help text for plugin.
    //
    std::wstring Plugin::HelpText() const
    {
        return L"";
    }

    //
    // Returns whether the plugin should be enabled or not in the
    // contextual menu. If not overridden, it will always be enabled.
    //
    // @param p_ParentPath Path of the parent directory of files that
    //                     triggered the contextual menu.
    // @param p_File Path of one file that was selected.
    // @returns true if plugin should be enabled in contextual menu.
    //
    bool Plugin::Enabled(const std::wstring& /*p_ParentPath*/,
                         const std::wstring& /*p_File*/) const
    {
        return true;
    }

    //
    // Checks if this plugin is a separator.
    // Do not override: will be overridden by the PluginSeparator class *only*.
    //
    // @return true if this plugin is a submenu separator proxy.
    //
    bool Plugin::IsSeparator() const
    {
        return false;
    }

}   // namespace PCC
