﻿// DefaultPlugins.cs
// (c) 2011, Charles Lechasseur
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System.Collections.Generic;
using PathCopyCopy.Settings.Properties;

namespace PCC
{
    //************************************************************************
    /// <summary>
    /// Helper to get a list of all default plugins in Path Copy Copy.
    /// </summary>
    //************************************************************************
    public static class DefaultPlugins
    {
        //************************************************************************
        /// <summary>
        /// Returns a list of all default plugins in Path Copy Copy.
        /// </summary>
        /// <returns>List of default plugins</returns>
        //************************************************************************
        public static List<Plugin> GetDefaultPlugins()
        {
            List<Plugin> plugins = new List<Plugin>();
            SeparatorPlugin separator = new SeparatorPlugin();

            // This mimics the code in PathCopyCopyPluginRegistration.cpp in the C++ project.

            plugins.Add(new Plugin(Resources.SHORT_NAME_PLUGIN_ID, Resources.SHORT_NAME_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.LONG_NAME_PLUGIN_ID, Resources.LONG_NAME_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.SHORT_PATH_PLUGIN_ID, Resources.SHORT_PATH_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.LONG_PATH_PLUGIN_ID, Resources.LONG_PATH_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.SHORT_FOLDER_PLUGIN_ID, Resources.SHORT_FOLDER_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.LONG_FOLDER_PLUGIN_ID, Resources.LONG_FOLDER_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.SHORT_UNC_PATH_PLUGIN_ID, Resources.SHORT_UNC_PATH_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.LONG_UNC_PATH_PLUGIN_ID, Resources.LONG_UNC_PATH_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.SHORT_UNC_FOLDER_PATH_PLUGIN_ID, Resources.SHORT_UNC_FOLDER_PATH_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.LONG_UNC_FOLDER_PATH_PLUGIN_ID, Resources.LONG_UNC_FOLDER_PATH_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.INTERNET_PATH_PLUGIN_ID, Resources.INTERNET_PATH_PLUGIN_DESCRIPTION));

            plugins.Add(separator);
            plugins.Add(new Plugin(Resources.UNIX_PATH_PLUGIN_ID, Resources.UNIX_PATH_PLUGIN_DESCRIPTION));
            plugins.Add(new Plugin(Resources.CYGWIN_PATH_PLUGIN_ID, Resources.CYGWIN_PATH_PLUGIN_DESCRIPTION));

            return plugins;
        }
    }
}
